@@ -11,7 +11,7 @@ use 5.00307;
 use Exporter;
 
 BEGIN {
-    $VERSION = '0.88';
+    $VERSION = '0.90';
     @EXPORT  = qw( Dump Load DumpFile LoadFile );
     @ISA     = qw( Exporter );
 
@@ -27,12 +27,31 @@ BEGIN {
         push @ISA, 'DynaLoader';
         __PACKAGE__->bootstrap($VERSION);
     };
+
 }
 
+use constant _QR_MAP => {
+    ''   => sub { qr/$_[0]/     },
+    i    => sub { qr/$_[0]/i    },
+    x    => sub { qr/$_[0]/x    },
+    ix   => sub { qr/$_[0]/ix   },
+    s    => sub { qr/$_[0]/s    },
+    is   => sub { qr/$_[0]/is   },
+    xs   => sub { qr/$_[0]/xs   },
+    ixs  => sub { qr/$_[0]/ixs  },
+    m    => sub { qr/$_[0]/m    },
+    im   => sub { qr/$_[0]/im   },
+    xm   => sub { qr/$_[0]/xm   },
+    ixm  => sub { qr/$_[0]/ixm  },
+    sm   => sub { qr/$_[0]/sm   },
+    ism  => sub { qr/$_[0]/ism  },
+    xsm  => sub { qr/$_[0]/xsm  },
+    ixsm => sub { qr/$_[0]/ixsm },
+};
+
 sub __qr_helper {
-    # XXX - Really bad idea - should split to MODIFIERS and REGEXP keys as per YAML.pm.
-    if (index($_[0], '(?-xism:') == 0) {
-        qr/${\substr($_[0], 8, -1)}/;
+    if ($_[0] =~ /\A  \(\?  ([ixsm]*)  -  (?:[ixsm]*)  : (.*) \)  \z/x) {
+        _QR_MAP->{$1}->($2);
     }
     else {
         qr/$_[0]/;
